﻿using System;
using System.ComponentModel;
using System.IO;
using System.Runtime.InteropServices.Automation;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Effects;
using System.Windows.Media.Imaging;

namespace ImageShop
{
    public partial class OutOfBrowserMainPage : UserControl
    {
        Window mainWindow = Application.Current.MainWindow;

        public OutOfBrowserMainPage()
        {
            InitializeComponent();
            mainWindow.Closing += new EventHandler<ClosingEventArgs>(MainWindow_Closing);

            InitializeHelpPage();
            
        }


        #region Features That Only Work Outside The Browser

        /*
         * Handling the window's Close event and allowing user to cancel close
         */
        void MainWindow_Closing(object sender, ClosingEventArgs e)
        {
            // This feature works in both sandboxed and trusted out-of-browser apps.

            if (e.IsCancelable) 
            {
                MessageBoxResult result = MessageBox.Show("Leaving already?", "Shutting down image|shop", MessageBoxButton.OKCancel);
                if (result == MessageBoxResult.Cancel)
                {

                    e.Cancel = true;

                    // optionally save state here...
                }
            }
        }


        /*
         *  Point a WebBrowser control to the Out-of-Browser apps' landing page on Microsoft.com
         */
        private void InitializeHelpPage()
        {
            Dispatcher.BeginInvoke(() =>
            {
                // This too works in both sandboxed and trusted out-of-browser apps

                WebBrowser wb = new WebBrowser();
                wb.Source = new Uri("http://www.microsoft.com/silverlight/resources/documentation/OutsideBrowser.aspx");
                wb.HorizontalAlignment = System.Windows.HorizontalAlignment.Stretch;
                wb.VerticalAlignment = System.Windows.VerticalAlignment.Stretch;

                helpPageHolder.Children.Add(wb);
            });
        }



        /*
         * When the app title is held down and dragged, drag-move the window
         */
        public void WindowDrag_MouseLeftButtonDown(object sender, MouseButtonEventArgs e)
        {
            // This works for sandboxed as well as trusted out-of-browser apps. 
            // With trusted apps it requires no user-initiation

            mainWindow.DragMove();

        }

        #endregion


        #region Features That Only Work in Trusted Apps

        /*
         * Open File btn click: Launch OpenFileDialog w/ filter set to image file types
         */
        private void openFileBtn_Click(object sender, RoutedEventArgs e)
        {
            // although this works in all apps (sandboxed/trusted and in/out of browser) 
            // in trusted apps this requires no user initiation
            // our current usage notwithstanding :)

            //OpenFileDialog ofd = new OpenFileDialog();
            //bool? retval = ofd.ShowDialog();
            //ofd.Filter = @"Image Files (*.bmp, *.jpg, *.png)|*.bmp;*.jpg;*.png";
            //if (retval != null && retval == true)
            //{
            //    AddImageToList(ofd.File);
            //}

            if (Application.Current.HasElevatedPermissions)
            {
                var imageFiles = Directory.EnumerateFiles(Environment.GetFolderPath(Environment.SpecialFolder.MyPictures), "*.jpg", SearchOption.AllDirectories);
                foreach (var imagePath in imageFiles)
                {
                    AddImageToList(new FileInfo(imagePath));
                }
            }
        }


        int BLANK_LAYOUT = 12;
        /*
         * Export btn click: Automate PowerPoint and create a slide deck
         * Specific automation APIs for PowerPoint are documented here:
         *      http://msdn.microsoft.com/en-us/library/bb265987.aspx
         */
        private void exportToPPT_Click(object sender, RoutedEventArgs e)
        {
            // query if the feature is available
            // ... makes for portable code
            if (AutomationFactory.IsAvailable)
            {
                dynamic powerpoint = AutomationFactory.CreateObject("PowerPoint.Application");
                powerpoint.Visible = true;


                // create a new presentation deck and activate the PowerPoint window
                dynamic presentation = powerpoint.Presentations.Add();
                powerpoint.ActiveWindow.Activate();

                foreach (Thumbnail thumbnail in listOfItems.Items)
                {
                    Image img = thumbnail.OriginalImage;
                    string path = (string)img.Tag;

                    // since PowerPoint expects the full file path...
                    if (!string.IsNullOrWhiteSpace(path) && System.IO.Path.IsPathRooted(path)) 
                    {
                        // add a slide to the deck
                        presentation.Slides.Add(presentation.Slides.Count + 1, BLANK_LAYOUT);

                        // insert picture into current slide
                        dynamic currentSlide = presentation.Slides(presentation.Slides.Count);
                        currentSlide.Shapes.AddPicture(path, false, true, 30, 30, 640, 480); // you can use actual img dimensions instead of 640x480
                    }

                }
            }
        }

        //
        // tip: also look inside SearchForFiles.xaml.cs
        //


        #endregion


        #region Other Silverlight Features
        /*
         *  Drop target logic: Add images to list
         */
        private void editCanvas_Drop(object sender, DragEventArgs e)
        {
            IDataObject data = e.Data;
            if (data.GetDataPresent(DataFormats.FileDrop))
            {
                FileInfo[] files = data.GetData(DataFormats.FileDrop) as FileInfo[];

                foreach (var file in files)
                {
                    AddImageToList(file);
                }
            }

        }


        /*
         *  Effects combo box selection changed: apply effect to img and refresh its thumbnail
         */
        private void fxCombo_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            Image imgBeingEdited = (Image)lytRoot.FindName("imgBeingEdited");
            if (imgBeingEdited != null)
            {
                (imgBeingEdited as Image).Effect = (((sender as ComboBox).SelectedItem as ComboBoxItem).Content as ShaderEffect);
                Thumbnail t = listOfItems.SelectedItem as Thumbnail;
                t.ThumbnailImage.Source = new WriteableBitmap(imgBeingEdited, null);
            }
        }




        #endregion


        #region HouseKeeping

        private void AddImageToList(FileInfo fileinfo)
        {
            FileStream fileStream = fileinfo.OpenRead();
            Image img = new Image();
            BitmapImage bi = new BitmapImage();
            bi.SetSource(fileStream);
            img.Margin = new Thickness(5d);
            img.Stretch = Stretch.UniformToFill;
            img.Source = bi;

            try { img.Tag = fileinfo.FullName; }
            catch { }

            Thumbnail thumbnail = new Thumbnail();
            thumbnail.OriginalImage = img;



            listOfItems.Items.Add(thumbnail);
        }

        private void listOfItems_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            ClearEditCanvas();
            var selectedItem = (sender as ListBox).SelectedItem;
            if (selectedItem != null)
            {
                Thumbnail t = selectedItem as Thumbnail;
                Image img = t.OriginalImage;
                img.Stretch = Stretch.Uniform;
                img.SetValue(Image.NameProperty, "imgBeingEdited");
                editCanvas.Children.Add(img);
            }
        }


        private void newFileBtn_Click(object sender, RoutedEventArgs e)
        {
            listOfItems.Items.Clear();
            editCanvas.Children.Clear();
            editCanvas.Background = new SolidColorBrush(Colors.Gray);
        }

        private void ClearEditCanvas()
        {
            for (int i = 0; i < editCanvas.Children.Count; i++)
            {
                editCanvas.Children.RemoveAt(i);
            }
        }

        private void searchBox_SearchResultsRetrieved(object sender, EventArgs eventArgs)
        {
            foreach (var file in searchBox.SearchResults)
            {
                AddImageToList(file);
            }
        }

        private void aboutBtn_Click(object sender, RoutedEventArgs e)
        {
            if (helpPageHolder.Visibility == Visibility.Visible)
            {
                helpPageHolder.Visibility = Visibility.Collapsed;
            }
            else
            {
                helpPageHolder.Visibility = Visibility.Visible;
            }
        }

        #endregion

    }
}
