﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Runtime.InteropServices.Automation;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Input;
using System.Windows.Media;

namespace ImageShop
{
    public partial class SearchForFiles : UserControl
    {
        public List<FileInfo> SearchResults { get; set; }

        public delegate void SearchResultsRetrievedHandler(object sender, EventArgs eventArgs);

        // notify interested parties on search results
        public event SearchResultsRetrievedHandler SearchResultsRetrieved;

        public SearchForFiles()
        {
            SearchResults = new List<FileInfo>();
            InitializeComponent();
        }

        #region Process Input
        
        private void searchBtn_Click(object sender, RoutedEventArgs e)
        {
            SearchFiles();
        }

        private void searchTxtBox_KeyUp(object sender, KeyEventArgs e)
        {
            if (string.IsNullOrWhiteSpace(searchTxtBox.Text.Trim()) )
            {
                searchTxtBox.Background = new SolidColorBrush(Colors.LightGray);
            }

            if (e.Key != Key.Enter)
            {
                return;
            }

            searchTxtBox.Background = new SolidColorBrush(Colors.Yellow);
            SearchFiles();
        }
        
        #endregion

        #region Interop with OS Search

        void SearchFiles()
        {
            if (!string.IsNullOrWhiteSpace(searchTxtBox.Text) )
                Dispatcher.BeginInvoke(new Action(AsyncSearchFiles));
        }

        //
        // We're interacting with Windows Search using the SQL syntax.
        // here's more on this approach: http://msdn.microsoft.com/en-us/library/bb231256(VS.85).aspx
        //
        void AsyncSearchFiles()
        {

            try
            {
                // Defensive coding: check if you *can* automate, then automate...
                if (AutomationFactory.IsAvailable)
                {
                    dynamic connection = AutomationFactory.CreateObject("ADODB.Connection");
                    dynamic recordset = AutomationFactory.CreateObject("ADODB.Recordset");

                    string connectionString = @"Provider=Search.CollatorDSO;Extended properties='Application=Windows';";
                    string query = @"SELECT System.ItemPathDisplay, System.Keywords FROM SYSTEMINDEX WHERE System.Kind = 'picture' AND CONTAINS (System.Keywords, '" + searchTxtBox.Text + "') AND NOT CONTAINS(System.ItemFolderPathDisplay, 'Recycle') AND NOT CONTAINS(System.ItemFolderPathDisplay, 'Mailbox')";
                    connection.Open(connectionString);
                    recordset.Open(query, connection);

                    try
                    {
                        recordset.MoveFirst();
                    }
                    catch
                    {
                        // an exception at this stage indicates EOF (i.e. no results found)
                        searchTxtBox.Background = new SolidColorBrush(Colors.Yellow);
                        searchTxtBox.Text = "No Results";

                        recordset.Close();
                        connection.Close();

                        return;
                    }
                    SearchResults.Clear();


                    while (!((bool)recordset.EOF))
                    {
                        // System.ItemUrl would have been better but FileInfo only takes paths, not URLs
                        dynamic item = recordset.Fields.Item("System.ItemPathDisplay");
                        string s = item.Value.ToString();

                        FileInfo fileInfo;
                        try
                        {
                            // perhaps for backward compat Windows returns "My Pictures" instead of "Pictures"
                            // the following is to strip the "My " token off that string before creating FileInfo...
                            // Yeah, I know :)
                            fileInfo = new FileInfo(s.Remove(s.IndexOf("My "), 3));
                            SearchResults.Add(fileInfo);
                        }
                        catch
                        {
                            // An exception here would mean the file
                            // was inaccessible, given Silverlight limits silent access
                            // to Documents/Pictures/Music/Videos folders everywhere
                            // so we just skip to the next record
                        }

                        recordset.MoveNext();

                    }

                    recordset.Close();
                    connection.Close();

                    // raise the event
                    if (SearchResultsRetrieved != null)
                    {
                        SearchResultsRetrieved(this, EventArgs.Empty);
                    }
                }
            }
            catch (Exception ex)
            {
                SearchResults.Clear();
                searchTxtBox.Text = ex.Message;
                searchTxtBox.Background = new SolidColorBrush(Colors.Red);
            }

            searchTxtBox.Background = new SolidColorBrush(Colors.Green);
        }

        #endregion

    }
}
