﻿using System;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using System.Collections.Generic;
using System.IO;
using System.Runtime.Serialization;
using System.IO.IsolatedStorage;
using System.Windows.Media.Imaging;
using System.Linq;

namespace TrustedApp
{
    [DataContract]
    public class MediaFile
    {
        [DataMember]
        public string ID { get; set; }
        [DataMember]
        public string Artist { get; set; }
        [DataMember]
        public string Title { get; set; }
        [DataMember]
        public string Album { get; set; }
        [DataMember]
        public string Genre { get; set; }
        [DataMember]
        public string AlbumArtPath { get; set; }

        private const string _cachePath = "mediaTags.cache";
        private BitmapImage _default = null;

        public ImageSource AlbumArtStream
        {
            get
            {
                BitmapImage image;

                if (string.IsNullOrEmpty(AlbumArtPath))
                {
                    if (null == _default)
                    {
                        _default = new BitmapImage(new Uri("../Images/default.png", UriKind.Relative));
                    }

                    image = _default;
                }
                else
                {
                    FileStream stream = File.Open(AlbumArtPath, FileMode.Open, FileAccess.Read);

                    image = new BitmapImage();
                    image.SetSource(stream);
                    stream.Close();
                }

                return image;
            }
        }

        public static List<MediaFile> Shuffle(List<MediaFile> list)
        {
            List<MediaFile> randomList = new List<MediaFile>();
            int idx = 0;
            Random rand = new Random();

            while (list.Count > 0)
            {
                idx = rand.Next(0, list.Count);
                randomList.Add(list[idx]);
                list.RemoveAt(idx);
            }

            return randomList;
        }

        public static List<MediaFile> GetMediaFiles()
        {
            List<MediaFile> files = null; ;
            MediaFile mf;
            string path = Environment.GetFolderPath(Environment.SpecialFolder.MyMusic);
            IEnumerable<string> list = Directory.EnumerateFiles(path, "*.mp3", SearchOption.AllDirectories);
            TagLib.File tags;

            // Check for cached tag file
            files = GetCachedList(list);

            // Check for cache
            if (null == files || files.Count == 0)
            {
                // Create new
                files = new List<MediaFile>();

                // Read files
                foreach (string file in list)
                {
                    mf = new MediaFile();

                    // Save path (ID)
                    mf.ID = file;
                    mf.AlbumArtPath = GetAlbumArtPath(file);

                    // Put in list
                    files.Add(mf);
                }

                for (int idx = 0; idx < files.Count; idx++)
                {
                    mf = files[idx];

                    // Get tags
                    tags = TagLib.File.Create(mf.ID);

                    // Set properties
                    mf.Artist = tags.Tag.FirstPerformer;
                    mf.Title = tags.Tag.Title;
                    mf.Album = tags.Tag.Album;
                    mf.Genre = tags.Tag.FirstGenre;
                }

                // Persist
                SaveCachedList(files);
            }

            // If no tag file, then generate
            return files;
        }

        public static string GetAlbumArtPath(string path)
        {
            int pos = path.LastIndexOf('\\');
            string root = path.Substring(0, pos + 1);
            IEnumerable<string> art;

            // Find item
            if ((art = Directory.EnumerateFiles(root, "*AlbumArt_*.jpg", SearchOption.TopDirectoryOnly)) != null)
            {
                root = art.GetEnumerator().Current;
            }
            else
            {
                root = null;
            }

            return root;
        }

        public static List<MediaFile> GetCachedList(IEnumerable<string> files)
        {
            List<MediaFile> tags;

            using (IsolatedStorageFile store = IsolatedStorageFile.GetUserStoreForApplication())
            {
                if (store.FileExists(_cachePath))
                {
                    using (IsolatedStorageFileStream fs = store.OpenFile(_cachePath, FileMode.Open, FileAccess.Read))
                    {
                        int count = 0;
                        DataContractSerializer ds = new DataContractSerializer(typeof(List<MediaFile>));
                        tags = ds.ReadObject(fs) as List<MediaFile>;

                        // Get count
                        foreach (string item in files) { count++; }

                        // If counts are different, then regenerate cache
                        if (count != tags.Count)
                        {
                            // Cache is dirty
                            tags = null;
                        }
                    }
                }
                else
                {
                    tags = null;
                }
            }

            return tags;
        }

        public static void SaveCachedList(List<MediaFile> tags)
        {
            using (IsolatedStorageFile store = IsolatedStorageFile.GetUserStoreForApplication())
            {
                using (IsolatedStorageFileStream fs = store.OpenFile(_cachePath, FileMode.Create, FileAccess.Write))
                {
                    DataContractSerializer ds = new DataContractSerializer(typeof(List<MediaFile>));
                    ds.WriteObject(fs, tags);
                }
            }
        }

    }
}
